@@ -64,9 +64,9 @@ static void help_cb(GtkAction* action, AppletData* data)
 {
 	GError* error = NULL;
 	char* uri;
-	#define NA_HELP_DOC "user-guide"
+	#define NA_HELP_DOC "mate-user-guide"
 
-	uri = g_strdup_printf("ghelp:%s?%s", NA_HELP_DOC, "panels-notification-area");
+	uri = g_strdup_printf("help:%s/%s", NA_HELP_DOC, "panels-notification-area");
 
 	gtk_show_uri(gtk_widget_get_screen(GTK_WIDGET(data->applet)), uri, gtk_get_current_event_time(), &error);
 
@@ -141,7 +141,11 @@ static const GtkActionEntry menu_actions [] = {
 	  G_CALLBACK (about_cb) }
 };
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBackgroundType type, GdkColor* color, cairo_pattern_t *pattern, AppletData* data)
+#else
 static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBackgroundType type, GdkColor* color, GdkPixmap* pixmap, AppletData* data)
+#endif
 {
 	na_tray_force_redraw(data->tray);
 }
