@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "linux" -*- */
 /*
- * libmatewnck based tasklist applet.
+ * libwnck based tasklist applet.
  * (C) 2001 Red Hat, Inc
  * (C) 2001 Alexander Larsson
  *
@@ -19,9 +19,15 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libmatewnck/libmatewnck.h>
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE
+#include <libwnck/libwnck.h>
 #include <gio/gio.h>
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define MATE_DESKTOP_USE_UNSTABLE_API
+#include <libmate-desktop/mate-desktop-utils.h>
+#endif
+
 #include "wncklet.h"
 #include "window-list.h"
 
@@ -33,7 +39,7 @@ typedef struct {
 	GtkWidget* tasklist;
 
 	gboolean include_all_workspaces;
-	MatewnckTasklistGroupingType grouping;
+	WnckTasklistGroupingType grouping;
 	gboolean move_unminimized_windows;
 
 	GtkOrientation orientation;
@@ -55,7 +61,7 @@ typedef struct {
 	GSettings* settings;
 } TasklistData;
 
-static void callSystemMonitor(GtkAction* action, TasklistData* tasklist);
+static void call_system_monitor(GtkAction* action, TasklistData* tasklist);
 static void display_properties_dialog(GtkAction* action, TasklistData* tasklist);
 static void display_help_dialog(GtkAction* action, TasklistData* tasklist);
 static void display_about_dialog(GtkAction* action, TasklistData* tasklist);
@@ -71,16 +77,16 @@ static void tasklist_update(TasklistData* tasklist)
 		gtk_widget_set_size_request(GTK_WIDGET(tasklist->tasklist), tasklist->size, -1);
 	}
 
-	matewnck_tasklist_set_grouping(MATEWNCK_TASKLIST(tasklist->tasklist), tasklist->grouping);
-	matewnck_tasklist_set_include_all_workspaces(MATEWNCK_TASKLIST(tasklist->tasklist), tasklist->include_all_workspaces);
-	matewnck_tasklist_set_switch_workspace_on_unminimize(MATEWNCK_TASKLIST(tasklist->tasklist), tasklist->move_unminimized_windows);
+	wnck_tasklist_set_grouping(WNCK_TASKLIST(tasklist->tasklist), tasklist->grouping);
+	wnck_tasklist_set_include_all_workspaces(WNCK_TASKLIST(tasklist->tasklist), tasklist->include_all_workspaces);
+	wnck_tasklist_set_switch_workspace_on_unminimize(WNCK_TASKLIST(tasklist->tasklist), tasklist->move_unminimized_windows);
 }
 
 static void response_cb(GtkWidget* widget, int id, TasklistData* tasklist)
 {
 	if (id == GTK_RESPONSE_HELP)
 	{
-		wncklet_display_help(widget, "user-guide", "windowlist-prefs", WINDOW_LIST_ICON);
+		wncklet_display_help(widget, "mate-user-guide", "windowlist-prefs", WINDOW_LIST_ICON);
 	}
 	else
 	{
@@ -115,20 +121,28 @@ static void applet_change_orient(MatePanelApplet* applet, MatePanelAppletOrient
 
 	tasklist->orientation = new_orient;
 
-	matewnck_tasklist_set_orientation (tasklist->tasklist, new_orient);
+#ifdef WNCK_CHECK_VERSION
+#if WNCK_CHECK_VERSION (3, 4, 6)
+	wnck_tasklist_set_orientation (tasklist->tasklist, new_orient);
+#endif
+#endif
 	tasklist_update(tasklist);
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBackgroundType type, GdkColor* color, cairo_pattern_t* pattern, TasklistData* tasklist)
+#else
 static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBackgroundType type, GdkColor* color, GdkPixmap* pixmap, TasklistData* tasklist)
+#endif
 {
 	switch (type)
 	{
 		case PANEL_NO_BACKGROUND:
-			matewnck_tasklist_set_button_relief(MATEWNCK_TASKLIST(tasklist->tasklist), GTK_RELIEF_NORMAL);
+			wnck_tasklist_set_button_relief(WNCK_TASKLIST(tasklist->tasklist), GTK_RELIEF_NORMAL);
 			break;
 		case PANEL_COLOR_BACKGROUND:
 		case PANEL_PIXMAP_BACKGROUND:
-			matewnck_tasklist_set_button_relief(MATEWNCK_TASKLIST(tasklist->tasklist), GTK_RELIEF_NONE);
+			wnck_tasklist_set_button_relief(WNCK_TASKLIST(tasklist->tasklist), GTK_RELIEF_NONE);
 			break;
 	}
 }
@@ -155,7 +169,7 @@ static void destroy_tasklist(GtkWidget* widget, TasklistData* tasklist)
 
 /* TODO: this is sad, should be used a function to retrieve  applications from
  *  .desktop or some like that. */
-static const char* listOfSystemMonitors[] = {
+static const char* system_monitors[] = {
 	"mate-system-monitor",
 	"gnome-system-monitor",
 };
@@ -167,7 +181,7 @@ static const GtkActionEntry tasklist_menu_actions[] = {
 		N_("_System Monitor"),
 		NULL,
 		NULL,
-		G_CALLBACK(callSystemMonitor)
+		G_CALLBACK(call_system_monitor)
 	},
 	{
 		"TasklistPreferences",
@@ -231,18 +245,18 @@ static void display_all_workspaces_changed(GSettings* settings, gchar* key, Task
 	tasklist_properties_update_content_radio(tasklist);
 }
 
-static GtkWidget* get_grouping_button(TasklistData* tasklist, MatewnckTasklistGroupingType type)
+static GtkWidget* get_grouping_button(TasklistData* tasklist, WnckTasklistGroupingType type)
 {
 	switch (type)
 	{
 		default:
-		case MATEWNCK_TASKLIST_NEVER_GROUP:
+		case WNCK_TASKLIST_NEVER_GROUP:
 			return tasklist->never_group_radio;
 			break;
-		case MATEWNCK_TASKLIST_AUTO_GROUP:
+		case WNCK_TASKLIST_AUTO_GROUP:
 			return tasklist->auto_group_radio;
 			break;
-		case MATEWNCK_TASKLIST_ALWAYS_GROUP:
+		case WNCK_TASKLIST_ALWAYS_GROUP:
 			return tasklist->always_group_radio;
 			break;
 	}
@@ -250,7 +264,7 @@ static GtkWidget* get_grouping_button(TasklistData* tasklist, MatewnckTasklistGr
 
 static void group_windows_changed(GSettings* settings, gchar* key, TasklistData* tasklist)
 {
-	MatewnckTasklistGroupingType type;
+	WnckTasklistGroupingType type;
 	GtkWidget* button;
 
 	type = g_settings_get_enum (settings, key);
@@ -317,16 +331,24 @@ static void setup_gsettings(TasklistData* tasklist)
 					  tasklist);
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void applet_size_allocate(GtkWidget *widget, GtkAllocation *allocation, TasklistData *tasklist)
+#else
 static void applet_size_request(GtkWidget* widget, GtkRequisition* requisition, TasklistData* tasklist)
+#endif
 {
 	int len;
 	const int* size_hints;
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GtkRequisition child_req;
-	MatewnckTasklist* matewncktl = MATEWNCK_TASKLIST(tasklist->tasklist);
+#endif
+	WnckTasklist* wncktl = WNCK_TASKLIST(tasklist->tasklist);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	gtk_widget_get_child_requisition(tasklist->applet, &child_req);
+#endif
 
-	size_hints = matewnck_tasklist_get_size_hint_list(matewncktl, &len);
+	size_hints = wnck_tasklist_get_size_hint_list(wncktl, &len);
 	g_assert(len % 2 == 0);
 
 	mate_panel_applet_set_size_hints(MATE_PANEL_APPLET(tasklist->applet), size_hints, len, 0);
@@ -413,15 +435,31 @@ gboolean window_list_applet_fill(MatePanelApplet* applet)
 			break;
 	}
 
-	tasklist->tasklist = matewnck_tasklist_new(NULL);
+#ifdef WNCK_CHECK_VERSION
+#if WNCK_CHECK_VERSION (2, 91, 6)
+	tasklist->tasklist = wnck_tasklist_new();
+#else
+	tasklist->tasklist = wnck_tasklist_new(NULL);
+#endif
+#else
+	tasklist->tasklist = wnck_tasklist_new(NULL);
+#endif
 
-	matewnck_tasklist_set_orientation (tasklist->tasklist, tasklist->orientation);
+#ifdef WNCK_CHECK_VERSION
+#if WNCK_CHECK_VERSION (3, 4, 6)
+	wnck_tasklist_set_orientation (tasklist->tasklist, tasklist->orientation);
+#endif
+#endif
 
-	matewnck_tasklist_set_icon_loader(MATEWNCK_TASKLIST(tasklist->tasklist), icon_loader_func, tasklist, NULL);
+	wnck_tasklist_set_icon_loader(WNCK_TASKLIST(tasklist->tasklist), icon_loader_func, tasklist, NULL);
 
 	g_signal_connect(G_OBJECT(tasklist->tasklist), "destroy", G_CALLBACK(destroy_tasklist), tasklist);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	g_signal_connect(G_OBJECT(tasklist->applet), "size_allocate", G_CALLBACK(applet_size_allocate), tasklist);
+#else
 	g_signal_connect(G_OBJECT(tasklist->applet), "size_request", G_CALLBACK(applet_size_request), tasklist);
+#endif
 	tasklist_update(tasklist);
 	gtk_widget_show(tasklist->tasklist);
 
@@ -444,9 +482,9 @@ gboolean window_list_applet_fill(MatePanelApplet* applet)
 	char* programpath;
 	int i;
 
-	for (i = 0; i < G_N_ELEMENTS(listOfSystemMonitors); i += 1)
+	for (i = 0; i < G_N_ELEMENTS(system_monitors); i += 1)
 	{
-		programpath = g_find_program_in_path(listOfSystemMonitors[i]);
+		programpath = g_find_program_in_path(system_monitors[i]);
 		
 		if (programpath != NULL)
 		{
@@ -484,26 +522,33 @@ gboolean window_list_applet_fill(MatePanelApplet* applet)
 	return TRUE;
 }
 
-static void callSystemMonitor(GtkAction* action, TasklistData* tasklist)
+static void call_system_monitor(GtkAction* action, TasklistData* tasklist)
 {
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	char* argv[2] = {NULL, NULL};
+#endif
 	char* programpath;
 	int i;
 
-	for (i = 0; i < G_N_ELEMENTS(listOfSystemMonitors); i += 1)
+	for (i = 0; i < G_N_ELEMENTS(system_monitors); i += 1)
 	{
-		programpath = g_find_program_in_path(listOfSystemMonitors[i]);
-		
+		programpath = g_find_program_in_path(system_monitors[i]);
+
 		if (programpath != NULL)
 		{
 			g_free(programpath);
-			
-			argv[0] = listOfSystemMonitors[i];
-			
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+			mate_gdk_spawn_command_line_on_screen(gtk_widget_get_screen(tasklist->applet),
+				      system_monitors[i],
+				      NULL);
+#else
+			argv[0] = system_monitors[i];
 			gdk_spawn_on_screen(gtk_widget_get_screen(tasklist->applet), NULL, argv, NULL,
 				      G_SPAWN_SEARCH_PATH,
 				      NULL, NULL, NULL, NULL);
-			
+#endif
+
 			return;
 		}
 	}
@@ -512,7 +557,7 @@ static void callSystemMonitor(GtkAction* action, TasklistData* tasklist)
 
 static void display_help_dialog(GtkAction* action, TasklistData* tasklist)
 {
-	wncklet_display_help(tasklist->applet, "user-guide", "windowlist", WINDOW_LIST_ICON);
+	wncklet_display_help(tasklist->applet, "mate-user-guide", "windowlist", WINDOW_LIST_ICON);
 }
 
 static void display_about_dialog(GtkAction* action, TasklistData* tasklist)
