@@ -33,10 +33,15 @@
 
 #include <glib/gi18n-lib.h>
 #include <cairo.h>
+#include <cairo-xlib.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
+#if GTK_CHECK_VERSION (3, 0, 0)
+#include <gtk/gtkx.h>
+#include <gdk/gdkkeysyms-compat.h>
+#endif
 #include <X11/Xatom.h>
 
 #include "mate-panel-applet.h"
@@ -84,7 +89,6 @@ enum {
 	CHANGE_SIZE,
 	CHANGE_BACKGROUND,
 	MOVE_FOCUS_OUT_OF_APPLET,
-	SAVE_YOURSELF,
 	LAST_SIGNAL
 };
 
@@ -557,7 +561,11 @@ mate_panel_applet_request_focus (MatePanelApplet	 *applet,
 	display = gdk_screen_get_display (screen);
 
 	xdisplay = GDK_DISPLAY_XDISPLAY (display);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	xroot	 = GDK_WINDOW_XID (root);
+#else
 	xroot	 = GDK_WINDOW_XWINDOW (root);
+#endif
 
 	mate_panel_applet_init_atoms (xdisplay);
 
@@ -792,6 +800,9 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 	MatePanelApplet    *applet;
 	GtkAllocation   allocation;
 	GtkRequisition  requisition;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkDevice      *device;
+#endif
 	GdkScreen      *screen;
 	int             menu_x = 0;
 	int             menu_y = 0;
@@ -802,13 +813,26 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 
 	applet = MATE_PANEL_APPLET (widget);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	screen = gtk_widget_get_screen (widget);
+	gtk_menu_set_screen (menu, screen);
+#else
 	screen = gtk_window_get_screen (GTK_WINDOW (applet->priv->plug));
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
+#else
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+#endif
 	gdk_window_get_origin (gtk_widget_get_window (widget),
 			       &menu_x, &menu_y);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	device = gdk_device_manager_get_client_pointer (gdk_display_get_device_manager (gtk_widget_get_display (widget)));
+	gdk_window_get_device_position(gtk_widget_get_window (widget), device, &pointer_x, &pointer_y, NULL);
+#else
 	gtk_widget_get_pointer (widget, &pointer_x, &pointer_y);
-
+#endif
 	gtk_widget_get_allocation (widget, &allocation);
 
 	menu_x += allocation.x;
@@ -850,7 +874,11 @@ mate_panel_applet_position_menu (GtkMenu   *menu,
 
 	*x = menu_x;
 	*y = menu_y;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	*push_in = FALSE;
+#else
 	*push_in = TRUE;
+#endif
 }
 
 static void
@@ -908,17 +936,27 @@ mate_panel_applet_button_event (GtkWidget      *widget,
 		 * selected.
 		 * We don't want to hog the pointer on our parent.
 		 */
+#if GTK_CHECK_VERSION (3, 0, 0)
+		gdk_device_ungrab (event->device, GDK_CURRENT_TIME);
+#else
 		gdk_display_pointer_ungrab
 			(gtk_widget_get_display (widget),
 			 GDK_CURRENT_TIME);
+#endif
 	} else {
 		xevent.xbutton.type = ButtonRelease;
 	}
 
 	xevent.xbutton.display     = GDK_WINDOW_XDISPLAY (window);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	xevent.xbutton.window      = GDK_WINDOW_XID (socket_window);
+	xevent.xbutton.root        = GDK_WINDOW_XID (gdk_screen_get_root_window
+							 (gdk_window_get_screen (window)));
+#else
 	xevent.xbutton.window      = GDK_WINDOW_XWINDOW (socket_window);
 	xevent.xbutton.root        = GDK_WINDOW_XWINDOW (gdk_screen_get_root_window
 							 (gdk_drawable_get_screen (window)));
+#endif
 	/*
 	 * FIXME: the following might cause
 	 *        big problems for non-GTK apps
@@ -934,11 +972,19 @@ mate_panel_applet_button_event (GtkWidget      *widget,
 	gdk_error_trap_push ();
 
 	XSendEvent (GDK_WINDOW_XDISPLAY (window),
+#if GTK_CHECK_VERSION (3, 0, 0)
+		    GDK_WINDOW_XID (socket_window),
+#else
 		    GDK_WINDOW_XWINDOW (socket_window),
+#endif
 		    False, NoEventMask, &xevent);
 
 	gdk_flush ();
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_error_trap_pop_ignored ();
+#else
 	gdk_error_trap_pop ();
+#endif
 
 	return TRUE;
 }
@@ -982,13 +1028,76 @@ mate_panel_applet_popup_menu (GtkWidget *widget)
 	return TRUE;
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void
+mate_panel_applet_get_preferred_width (GtkWidget *widget,
+				       int       *minimum_width,
+				       int       *natural_width)
+{
+	int focus_width = 0;
+
+	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->get_preferred_width (widget,
+										minimum_width,
+										natural_width);
+	if (!mate_panel_applet_can_focus (widget))
+		return;
+
+	/* We are deliberately ignoring focus-padding here to
+	 * save valuable panel real estate.
+	 */
+	gtk_widget_style_get (widget,
+			      "focus-line-width", &focus_width,
+			      NULL);
+
+	*minimum_width += 2 * focus_width;
+	*natural_width += 2 * focus_width;
+}
+
+static void
+mate_panel_applet_get_preferred_height (GtkWidget *widget,
+					int       *minimum_height,
+					int       *natural_height)
+{
+	int focus_width = 0;
+
+	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->get_preferred_height (widget,
+										 minimum_height,
+										 natural_height);
+	if (!mate_panel_applet_can_focus (widget))
+		return;
+
+	/* We are deliberately ignoring focus-padding here to
+	 * save valuable panel real estate.
+	 */
+	gtk_widget_style_get (widget,
+			      "focus-line-width", &focus_width,
+			      NULL);
+
+	*minimum_height += 2 * focus_width;
+	*natural_height += 2 * focus_width;
+}
+
+static GtkSizeRequestMode
+mate_panel_applet_get_request_mode (GtkWidget *widget)
+{
+	MatePanelApplet *applet = MATE_PANEL_APPLET (widget);
+	MatePanelAppletOrient orientation;
+
+	orientation = mate_panel_applet_get_orient (applet);
+	if (orientation == MATE_PANEL_APPLET_ORIENT_UP ||
+	    orientation == MATE_PANEL_APPLET_ORIENT_DOWN)
+		return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
+
+	return GTK_SIZE_REQUEST_WIDTH_FOR_HEIGHT;
+}
+#else
 static void
 mate_panel_applet_size_request (GtkWidget *widget, GtkRequisition *requisition)
 {
 	int focus_width = 0;
 
 	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->size_request (widget,
-								    requisition);
+									 requisition);
 
 	if (!mate_panel_applet_can_focus (widget))
 		return;
@@ -1004,6 +1113,7 @@ mate_panel_applet_size_request (GtkWidget *widget, GtkRequisition *requisition)
 	requisition->width  += 2 * focus_width;
 	requisition->height += 2 * focus_width;
 }
+#endif
 
 static void
 mate_panel_applet_size_allocate (GtkWidget     *widget,
@@ -1064,22 +1174,45 @@ mate_panel_applet_size_allocate (GtkWidget     *widget,
 	}
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static gboolean mate_panel_applet_draw(GtkWidget* widget, cairo_t* cr)
+#else
 static gboolean mate_panel_applet_expose(GtkWidget* widget, GdkEventExpose* event)
+#endif
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkStyleContext *context;
+#else
 	GtkAllocation allocation;
+#endif
 	int border_width;
 	int focus_width = 0;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdouble x, y, width, height;
+#else
 	int x, y, width, height;
+#endif
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	g_return_val_if_fail (PANEL_IS_APPLET (widget), FALSE);
 	g_return_val_if_fail (event != NULL, FALSE);
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->draw(widget, cr);
+#else
 	GTK_WIDGET_CLASS (mate_panel_applet_parent_class)->expose_event(widget, event);
+#endif
 
         if (!gtk_widget_has_focus (widget))
 		return FALSE;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	width = gtk_widget_get_allocated_width (widget);
+	height = gtk_widget_get_allocated_height (widget);
+#else
 	gtk_widget_get_allocation(widget, &allocation);
+#endif
 
 	/*
 	 * We are deliberately ignoring focus-padding here to
@@ -1091,6 +1224,22 @@ static gboolean mate_panel_applet_expose(GtkWidget* widget, GdkEventExpose* even
 
 	border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	x = 0;
+	y = 0;
+
+	width -= 2 * border_width;
+	height -= 2 * border_width;
+
+	context = gtk_widget_get_style_context (widget);
+	gtk_style_context_save (context);
+
+	cairo_save (cr);
+	gtk_render_focus (context, cr, x, y, width, height);
+	cairo_restore (cr);
+
+	gtk_style_context_restore (context);
+#else
 	x = allocation.x;
 	y = allocation.y;
 
@@ -1100,8 +1249,10 @@ static gboolean mate_panel_applet_expose(GtkWidget* widget, GdkEventExpose* even
 	gtk_paint_focus (gtk_widget_get_style (widget),
 			 gtk_widget_get_window (widget),
 			 gtk_widget_get_state (widget),
-			 &event->area, widget, "mate_panel_applet",
+			 &event->area,
+			 widget, "mate_panel_applet",
 			 x, y, width, height);
+#endif
 
 	return FALSE;
 }
@@ -1156,12 +1307,21 @@ mate_panel_applet_focus (GtkWidget        *widget,
 
 static gboolean
 mate_panel_applet_parse_color (const gchar *color_str,
-			  GdkColor    *color)
+#if GTK_CHECK_VERSION (3, 0, 0)
+			       GdkRGBA     *color)
+#else
+			       GdkColor    *color)
+#endif
 {
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	int r, g, b;
+#endif
 
 	g_assert (color_str && color);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	return gdk_rgba_parse (color, color_str);
+#else
 	if (sscanf (color_str, "%4x%4x%4x", &r, &g, &b) != 3)
 		return FALSE;
 
@@ -1170,11 +1330,16 @@ mate_panel_applet_parse_color (const gchar *color_str,
 	color->blue  = b;
 
 	return TRUE;
+#endif
 }
 
 static gboolean
 mate_panel_applet_parse_pixmap_str (const char *str,
+#if GTK_CHECK_VERSION (3, 0, 0)
+			       Window          *xid,
+#else
 			       GdkNativeWindow *xid,
+#endif
 			       int             *x,
 			       int             *y)
 {
@@ -1208,24 +1373,57 @@ mate_panel_applet_parse_pixmap_str (const char *str,
 	if (tmp == elements [2])
 		goto ERROR_AND_FREE;
 
- 	g_strfreev (elements);
+	g_strfreev (elements);
 	return TRUE;
 
- ERROR_AND_FREE:
- 	g_strfreev (elements);
+ERROR_AND_FREE:
+	g_strfreev (elements);
 	return FALSE;
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static cairo_surface_t *
+mate_panel_applet_create_foreign_surface_for_display (GdkDisplay *display,
+                                                      GdkVisual  *visual,
+                                                      Window      xid)
+{
+        Window window;
+        gint x, y;
+        guint width, height, border, depth;
+
+        if (!XGetGeometry (GDK_DISPLAY_XDISPLAY (display), xid, &window,
+                           &x, &y, &width, &height, &border, &depth))
+                return NULL;
+
+        return cairo_xlib_surface_create (GDK_DISPLAY_XDISPLAY (display),
+                                          xid, gdk_x11_visual_get_xvisual (visual),
+                                          width, height);
+}
+
+static cairo_pattern_t *
+mate_panel_applet_get_pattern_from_pixmap (MatePanelApplet *applet,
+#else
 static GdkPixmap *
 mate_panel_applet_get_pixmap (MatePanelApplet     *applet,
+#endif
+#if GTK_CHECK_VERSION (3, 0, 0)
+			 Window           xid,
+#else
 			 GdkNativeWindow  xid,
+#endif
 			 int              x,
 			 int              y)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	cairo_surface_t *background;
+	cairo_surface_t *surface;
+	cairo_matrix_t   matrix;
+#else
 	gboolean         display_grabbed;
 	GdkPixmap       *pixmap;
 	GdkDisplay      *display;
 	GdkPixmap       *retval;
+#endif
 	GdkWindow       *window;
 	int              width;
 	int              height;
@@ -1237,11 +1435,28 @@ mate_panel_applet_get_pixmap (MatePanelApplet     *applet,
 	if (!gtk_widget_get_realized (GTK_WIDGET (applet)))
 		return NULL;
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	display = gdk_display_get_default ();
 	display_grabbed = FALSE;
+#endif
 
 	window = gtk_widget_get_window (GTK_WIDGET (applet));
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_error_trap_push ();
+	background = mate_panel_applet_create_foreign_surface_for_display (gdk_window_get_display (window),
+									   gdk_window_get_visual (window),
+									   xid);
+	gdk_error_trap_pop_ignored ();
+
+	/* background can be NULL if the user changes the background very fast.
+	* We'll get the next update, so it's not a big deal. */
+	if (!background || cairo_surface_status (background) != CAIRO_STATUS_SUCCESS) {
+		if (background)
+			cairo_surface_destroy (background);
+		return NULL;
+	}
+#else
 	pixmap = gdk_pixmap_lookup_for_display (display, xid);
 	if (pixmap)
 		g_object_ref (pixmap);
@@ -1258,20 +1473,50 @@ mate_panel_applet_get_pixmap (MatePanelApplet     *applet,
 			gdk_x11_display_ungrab (display);
 		return NULL;
 	}
+#endif
 
-	#if GTK_CHECK_VERSION(3, 0, 0)
-		width = gdk_window_get_width(window);
-		height = gdk_window_get_height(window);
-	#else
-		gdk_drawable_get_size(GDK_DRAWABLE(window), &width, &height);
-	#endif
+#if GTK_CHECK_VERSION(3, 0, 0)
+	width = gdk_window_get_width(window);
+	height = gdk_window_get_height(window);
+	surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24, width, height);
+#else
+	gdk_drawable_get_size(GDK_DRAWABLE(window), &width, &height);
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_error_trap_push ();
+	cr = cairo_create (surface);
+	cairo_set_source_surface (cr, background, -x, -y);
+	cairo_rectangle (cr, 0, 0, width, height);
+	cairo_fill (cr);
+	gdk_error_trap_pop_ignored ();
+#else
 	retval = gdk_pixmap_new (window, width, height, -1);
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	cairo_surface_destroy (background);
+	pattern = NULL;
+#else
 	/* the pixmap has no colormap, and we need one */
 	gdk_drawable_set_colormap (GDK_DRAWABLE (pixmap),
 				   gdk_drawable_get_colormap (window));
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	if (cairo_status (cr) == CAIRO_STATUS_SUCCESS) {
+		pattern = cairo_pattern_create_for_surface (surface);
+		cairo_matrix_init_translate (&matrix, 0, 0);
+		cairo_matrix_scale (&matrix, width, height);
+		cairo_pattern_set_matrix (pattern, &matrix);
+		cairo_pattern_set_extend (pattern, CAIRO_EXTEND_PAD);
+	}
+
+	cairo_destroy (cr);
+	cairo_surface_destroy (surface);
+
+	return pattern;
+#else
 	cr = gdk_cairo_create (GDK_DRAWABLE (retval));
 	gdk_cairo_set_source_pixmap (cr, pixmap, -x, -y);
 	pattern = cairo_get_source (cr);
@@ -1288,15 +1533,21 @@ mate_panel_applet_get_pixmap (MatePanelApplet     *applet,
 		gdk_x11_display_ungrab (display);
 
 	return retval;
+#endif
 }
 
 static MatePanelAppletBackgroundType
 mate_panel_applet_handle_background_string (MatePanelApplet  *applet,
-				       GdkColor     *color,
-				       GdkPixmap   **pixmap)
+#if GTK_CHECK_VERSION (3, 0, 0)
+					    GdkRGBA          *color,
+					    cairo_pattern_t **pattern)
+#else
+					    GdkColor         *color,
+					    GdkPixmap       **pixmap)
+#endif
 {
 	MatePanelAppletBackgroundType   retval;
-	char                      **elements;
+	char                          **elements;
 
 	retval = PANEL_NO_BACKGROUND;
 
@@ -1321,10 +1572,18 @@ mate_panel_applet_handle_background_string (MatePanelApplet  *applet,
 		retval = PANEL_COLOR_BACKGROUND;
 
 	} else if (elements [0] && !strcmp (elements [0], "pixmap")) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+		Window pixmap_id;
+#else
 		GdkNativeWindow pixmap_id;
+#endif
 		int             x, y;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+		g_return_val_if_fail (pattern != NULL, PANEL_NO_BACKGROUND);
+#else
 		g_return_val_if_fail (pixmap != NULL, PANEL_NO_BACKGROUND);
+#endif
 
 		if (!mate_panel_applet_parse_pixmap_str (elements [1], &pixmap_id, &x, &y)) {
 			g_warning ("Incomplete '%s' background type received: %s",
@@ -1334,12 +1593,21 @@ mate_panel_applet_handle_background_string (MatePanelApplet  *applet,
 			return PANEL_NO_BACKGROUND;
 		}
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+		*pattern = mate_panel_applet_get_pattern_from_pixmap (applet, pixmap_id, x, y);
+		if (!*pattern) {
+			g_warning ("Failed to get pattern %s", elements [1]);
+			g_strfreev (elements);
+			return PANEL_NO_BACKGROUND;
+		}
+#else
 		*pixmap = mate_panel_applet_get_pixmap (applet, pixmap_id, x, y);
 		if (!*pixmap) {
 			g_warning ("Failed to get pixmap %s", elements [1]);
 			g_strfreev (elements);
 			return PANEL_NO_BACKGROUND;
 		}
+#endif
 
 		retval = PANEL_PIXMAP_BACKGROUND;
 	} else
@@ -1351,19 +1619,37 @@ mate_panel_applet_handle_background_string (MatePanelApplet  *applet,
 }
 
 MatePanelAppletBackgroundType
-mate_panel_applet_get_background (MatePanelApplet *applet,
-			     GdkColor *color,
-			     GdkPixmap **pixmap)
+mate_panel_applet_get_background (MatePanelApplet  *applet,
+#if GTK_CHECK_VERSION (3, 0, 0)
+				  GdkRGBA          *color,
+				  cairo_pattern_t **pattern)
+#else
+				  GdkColor         *color,
+				  GdkPixmap       **pixmap)
+#endif
 {
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), PANEL_NO_BACKGROUND);
 
 	/* initial sanity */
+#if GTK_CHECK_VERSION (3, 0, 0)
+	if (pattern != NULL)
+		*pattern = NULL;
+#else
 	if (pixmap != NULL)
 		*pixmap = NULL;
+#endif
 	if (color != NULL)
+#if GTK_CHECK_VERSION (3, 0, 0)
+		memset (color, 0, sizeof (GdkRGBA));
+#else
 		memset (color, 0, sizeof (GdkColor));
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	return mate_panel_applet_handle_background_string (applet, color, pattern);
+#else
 	return mate_panel_applet_handle_background_string (applet, color, pixmap);
+#endif
 }
 
 static void
@@ -1384,45 +1670,126 @@ mate_panel_applet_set_background_string (MatePanelApplet *applet,
 	g_object_notify (G_OBJECT (applet), "background");
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static GtkStyleProperties *
+_mate_panel_applet_get_widget_style_properties (GtkWidget *widget, gboolean create_if_needed)
+{
+	GtkStyleProperties *properties;
+
+	properties = g_object_get_data (G_OBJECT (widget), "panel-applet-style-props");
+	if (!properties && create_if_needed) {
+		properties = gtk_style_properties_new ();
+		g_object_set_data_full (G_OBJECT (widget), "panel-applet-style-props",
+					properties, (GDestroyNotify) g_object_unref);
+	}
+	return properties;
+}
+
+static void
+_mate_panel_applet_reset_widget_style_properties (GtkWidget *widget)
+{
+	GtkStyleProperties *properties;
+
+	properties = _mate_panel_applet_get_widget_style_properties (widget, FALSE);
+
+	if (properties)
+		gtk_style_context_remove_provider (gtk_widget_get_style_context (widget),
+						   GTK_STYLE_PROVIDER (properties));
+
+	g_object_set_data (G_OBJECT (widget), "panel-applet-style-props", NULL);
+}
+#endif
+
 static void
 mate_panel_applet_update_background_for_widget (GtkWidget                 *widget,
 					   MatePanelAppletBackgroundType  type,
+#if GTK_CHECK_VERSION (3, 0, 0)
+					   GdkRGBA                   *color,
+					   cairo_pattern_t           *pattern)
+#else
 					   GdkColor                  *color,
 					   GdkPixmap                 *pixmap)
+#endif
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkStyleProperties *properties;
+#else
 	GtkRcStyle *rc_style;
 	GtkStyle   *style;
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_reset_style (widget);
+	if (!pattern) {
+		_mate_panel_applet_reset_widget_style_properties (widget);
+		return;
+	}
+	properties = _mate_panel_applet_get_widget_style_properties (widget, TRUE);
+#else
 	/* reset style */
 	gtk_widget_set_style (widget, NULL);
 	rc_style = gtk_rc_style_new ();
 	gtk_widget_modify_style (widget, rc_style);
 	g_object_unref (rc_style);
+#endif
 
 	switch (type) {
 	case PANEL_NO_BACKGROUND:
 		break;
 	case PANEL_COLOR_BACKGROUND:
+#if GTK_CHECK_VERSION (3, 0, 0)
+		gtk_style_properties_set (properties, GTK_STATE_FLAG_NORMAL,
+					  "background-color", &color,
+					  "background-image", NULL,
+					  NULL);
+#else
 		gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, color);
+#endif
 		break;
 	case PANEL_PIXMAP_BACKGROUND:
+#if GTK_CHECK_VERSION (3, 0, 0)
+		gtk_style_properties_set (properties, GTK_STATE_FLAG_NORMAL,
+					  /* background-color can't be NULL,
+					   * but is ignored anyway */
+					  "background-image", pattern,
+					  NULL);
+#else
 		style = gtk_style_copy (gtk_widget_get_style (widget));
 		if (style->bg_pixmap[GTK_STATE_NORMAL])
 			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
 		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
 		gtk_widget_set_style (widget, style);
 		g_object_unref (style);
+#endif
 		break;
 	default:
 		g_assert_not_reached ();
 		break;
 	}
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+	/* Note: this actually replaces the old properties, since it's the same
+	 * pointer */
+	gtk_style_context_add_provider (gtk_widget_get_style_context (widget),
+					GTK_STYLE_PROVIDER (properties),
+					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+#endif
 }
 
 static void
 mate_panel_applet_handle_background (MatePanelApplet *applet)
 {
 	MatePanelAppletBackgroundType  type;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkRGBA                    color;
+	cairo_pattern_t           *pattern;
+
+	type = mate_panel_applet_get_background (applet, &color, &pattern);
+
+	if (applet->priv->background_widget)
+		mate_panel_applet_update_background_for_widget (applet->priv->background_widget,
+							   type, &color, pattern);
+#else
 	GdkColor                   color;
 	GdkPixmap                 *pixmap;
 
@@ -1431,6 +1798,7 @@ mate_panel_applet_handle_background (MatePanelApplet *applet)
 	if (applet->priv->background_widget)
 		mate_panel_applet_update_background_for_widget (applet->priv->background_widget,
 							   type, &color, pixmap);
+#endif
 
 	switch (type) {
 	case PANEL_NO_BACKGROUND:
@@ -1446,9 +1814,17 @@ mate_panel_applet_handle_background (MatePanelApplet *applet)
 	case PANEL_PIXMAP_BACKGROUND:
 		g_signal_emit (G_OBJECT (applet),
 			       mate_panel_applet_signals [CHANGE_BACKGROUND],
+#if GTK_CHECK_VERSION (3, 0, 0)
+			       0, PANEL_PIXMAP_BACKGROUND, NULL, pattern);
+#else
 			       0, PANEL_PIXMAP_BACKGROUND, NULL, pixmap);
+#endif
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+		cairo_pattern_destroy (pattern);
+#else
 		g_object_unref (pixmap);
+#endif
 		break;
 	default:
 		g_assert_not_reached ();
@@ -1707,7 +2083,9 @@ static void
 mate_panel_applet_class_init (MatePanelAppletClass *klass)
 {
 	GObjectClass   *gobject_class = (GObjectClass *) klass;
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GtkObjectClass *object_class = (GtkObjectClass *) klass;
+#endif
 	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
 	GtkBindingSet *binding_set;
 
@@ -1718,9 +2096,16 @@ mate_panel_applet_class_init (MatePanelAppletClass *klass)
 
 	widget_class->button_press_event = mate_panel_applet_button_press;
 	widget_class->button_release_event = mate_panel_applet_button_release;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	widget_class->get_request_mode = mate_panel_applet_get_request_mode;
+	widget_class->get_preferred_width = mate_panel_applet_get_preferred_width;
+	widget_class->get_preferred_height = mate_panel_applet_get_preferred_height;
+	widget_class->draw = mate_panel_applet_draw;
+#else
 	widget_class->size_request = mate_panel_applet_size_request;
-	widget_class->size_allocate = mate_panel_applet_size_allocate;
 	widget_class->expose_event = mate_panel_applet_expose;
+#endif
+	widget_class->size_allocate = mate_panel_applet_size_allocate;
 	widget_class->focus = mate_panel_applet_focus;
 	widget_class->realize = mate_panel_applet_realize;
 	widget_class->popup_menu = mate_panel_applet_popup_menu;
@@ -1845,7 +2230,11 @@ mate_panel_applet_class_init (MatePanelAppletClass *klass)
 			      3,
 			      PANEL_TYPE_MATE_PANEL_APPLET_BACKGROUND_TYPE,
 			      GDK_TYPE_COLOR,
+#if GTK_CHECK_VERSION (3, 0, 0)
+			      CAIRO_GOBJECT_TYPE_PATTERN);
+#else
 			      GDK_TYPE_PIXMAP);
+#endif
 
 	mate_panel_applet_signals [MOVE_FOCUS_OUT_OF_APPLET] =
                 g_signal_new ("move_focus_out_of_applet",
@@ -1859,7 +2248,11 @@ mate_panel_applet_class_init (MatePanelAppletClass *klass)
 			      1,
 			      GTK_TYPE_DIRECTION_TYPE);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	binding_set = gtk_binding_set_by_class (gobject_class);
+#else
 	binding_set = gtk_binding_set_by_class (object_class);
+#endif
 	add_tab_bindings (binding_set, 0, GTK_DIR_TAB_FORWARD);
 	add_tab_bindings (binding_set, GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
 	add_tab_bindings (binding_set, GDK_CONTROL_MASK, GTK_DIR_TAB_FORWARD);
@@ -2139,16 +2532,29 @@ mate_panel_applet_set_background_widget (MatePanelApplet *applet,
 {
 	applet->priv->background_widget = widget;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	if (widget && gtk_widget_get_realized (widget)) {
+#else
 	if (widget) {
+#endif
 		MatePanelAppletBackgroundType  type;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		GdkRGBA                    color;
+		cairo_pattern_t           *pattern;
+		type = mate_panel_applet_get_background (applet, &color, &pattern);
+		mate_panel_applet_update_background_for_widget (widget, type,
+							   &color, pattern);
+		if (type == PANEL_PIXMAP_BACKGROUND)
+			cairo_pattern_destroy (pattern);
+#else
 		GdkColor                   color;
 		GdkPixmap                 *pixmap;
-
 		type = mate_panel_applet_get_background (applet, &color, &pixmap);
 		mate_panel_applet_update_background_for_widget (widget, type,
 							   &color, pixmap);
 		if (type == PANEL_PIXMAP_BACKGROUND)
 			g_object_unref (pixmap);
+#endif
 	}
 }
 
