@@ -41,8 +41,11 @@
 #include "panel-menu-bar.h"
 
 static Atom atom_mate_panel_action            = None;
+static Atom atom_gnome_panel_action           = None;
 static Atom atom_mate_panel_action_main_menu  = None;
 static Atom atom_mate_panel_action_run_dialog = None;
+static Atom atom_gnome_panel_action_main_menu  = None;
+static Atom atom_gnome_panel_action_run_dialog = None;
 static Atom atom_mate_panel_action_kill_dialog = None;
 
 static void
@@ -98,24 +101,42 @@ panel_action_protocol_filter (GdkXEvent *gdk_xevent,
 {
 	GdkWindow *window;
 	GdkScreen *screen;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkDisplay *display;
+#endif
 	XEvent    *xevent = (XEvent *) gdk_xevent;
 
 	if (xevent->type != ClientMessage)
 		return GDK_FILTER_CONTINUE;
 
-	if (xevent->xclient.message_type != atom_mate_panel_action)
+	if ((xevent->xclient.message_type != atom_mate_panel_action) &&
+	   (xevent->xclient.message_type != atom_gnome_panel_action))
 		return GDK_FILTER_CONTINUE;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	screen = gdk_event_get_screen (event);
+	display = gdk_screen_get_display (screen);
+	window = gdk_x11_window_lookup_for_display (display, xevent->xclient.window);
+#else
 	window = gdk_window_lookup (xevent->xclient.window);
+	screen = gdk_drawable_get_screen (window);
+#endif
 	if (!window)
 		return GDK_FILTER_CONTINUE;
 
-	screen = gdk_drawable_get_screen (window);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	if (window != gdk_screen_get_root_window (screen))
+		return GDK_FILTER_CONTINUE;
+#endif
 
 	if (xevent->xclient.data.l [0] == atom_mate_panel_action_main_menu)
 		panel_action_protocol_main_menu (screen, xevent->xclient.data.l [1]);
 	else if (xevent->xclient.data.l [0] == atom_mate_panel_action_run_dialog)
 		panel_action_protocol_run_dialog (screen, xevent->xclient.data.l [1]);
+	else if (xevent->xclient.data.l [0] == atom_gnome_panel_action_main_menu)
+		panel_action_protocol_main_menu (screen, xevent->xclient.data.l [1]);
+	else if (xevent->xclient.data.l [0] == atom_gnome_panel_action_run_dialog)
+		panel_action_protocol_run_dialog (screen, xevent->xclient.data.l [1]);
 	else if (xevent->xclient.data.l [0] == atom_mate_panel_action_kill_dialog)
 		panel_action_protocol_kill_dialog (screen, xevent->xclient.data.l [1]);
 	else
@@ -128,17 +149,28 @@ void
 panel_action_protocol_init (void)
 {
 	GdkDisplay *display;
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GdkAtom     gdk_atom_mate_panel_action;
+	GdkAtom     gdk_atom_gnome_panel_action;
+#endif
 
 	display = gdk_display_get_default ();
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	gdk_atom_mate_panel_action =
 		gdk_atom_intern_static_string ("_MATE_PANEL_ACTION");
+	gdk_atom_gnome_panel_action =
+		gdk_atom_intern_static_string ("_GNOME_PANEL_ACTION");
+#endif
 
 	atom_mate_panel_action =
 		XInternAtom (GDK_DISPLAY_XDISPLAY (display),
 			     "_MATE_PANEL_ACTION",
 			     FALSE);
+	atom_gnome_panel_action =
+		XInternAtom (GDK_DISPLAY_XDISPLAY (display),
+			     "_GNOME_PANEL_ACTION",
+			     FALSE);
 	atom_mate_panel_action_main_menu =
 		XInternAtom (GDK_DISPLAY_XDISPLAY (display),
 			     "_MATE_PANEL_ACTION_MAIN_MENU",
@@ -147,12 +179,28 @@ panel_action_protocol_init (void)
 		XInternAtom (GDK_DISPLAY_XDISPLAY (display),
 			     "_MATE_PANEL_ACTION_RUN_DIALOG",
 			     FALSE);
+	atom_gnome_panel_action_main_menu =
+		XInternAtom (GDK_DISPLAY_XDISPLAY (display),
+			     "_GNOME_PANEL_ACTION_MAIN_MENU",
+			     FALSE);
+	atom_gnome_panel_action_run_dialog =
+		XInternAtom (GDK_DISPLAY_XDISPLAY (display),
+			     "_GNOME_PANEL_ACTION_RUN_DIALOG",
+			     FALSE);
 	atom_mate_panel_action_kill_dialog =
 		XInternAtom (GDK_DISPLAY_XDISPLAY (display),
 			     "_MATE_PANEL_ACTION_KILL_DIALOG",
 			     FALSE);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	/* We'll filter event sent on non-root windows later */
+	gdk_window_add_filter (NULL, panel_action_protocol_filter, NULL);
+#else
 	gdk_display_add_client_message_filter (
 		display, gdk_atom_mate_panel_action,
 		panel_action_protocol_filter, NULL);
+	gdk_display_add_client_message_filter (
+		display, gdk_atom_gnome_panel_action,
+		panel_action_protocol_filter, NULL);
+#endif
 }
