@@ -795,7 +795,7 @@ panel_addto_dialog_response (GtkWidget *widget_dialog,
 	switch (response_id) {
 	case GTK_RESPONSE_HELP:
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (dialog->addto_dialog)),
-				 "user-guide", "gospanel-15", NULL);
+				 "mate-user-guide", "gospanel-15", NULL);
 		break;
 
 	case PANEL_ADDTO_RESPONSE_ADD:
@@ -1036,7 +1036,9 @@ panel_addto_name_notify (GSettings        *settings,
 			 gchar            *key,
 			 PanelAddtoDialog *dialog)
 {
-	panel_addto_name_change (dialog, g_settings_get_string (settings, key));
+	gchar *name = g_settings_get_string (settings, key);
+	panel_addto_name_change (dialog, name);
+	g_free (name);
 }
 
 static gboolean
@@ -1223,7 +1225,9 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 {
 	PanelAddtoDialog *dialog;
 	GtkWidget *dialog_vbox;
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	GtkWidget *vbox;
+#endif
 	GtkWidget *inner_vbox;
 	GtkWidget *find_hbox;
 	GtkWidget *sw;
@@ -1258,27 +1262,40 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 			       GTK_STOCK_CLOSE,
 			       GTK_RESPONSE_CLOSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), FALSE);
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog->addto_dialog),
 				      FALSE);
+#endif
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->addto_dialog),
 					 PANEL_ADDTO_RESPONSE_ADD);
 
 	gtk_container_set_border_width (GTK_CONTAINER (dialog->addto_dialog), 5);
 
 	dialog_vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog->addto_dialog));
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_box_set_spacing (GTK_BOX (dialog_vbox), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), 5);
+#else
 	gtk_box_set_spacing (GTK_BOX (dialog_vbox), 2);
+#endif
 
 	g_signal_connect (G_OBJECT (dialog->addto_dialog), "response",
 			  G_CALLBACK (panel_addto_dialog_response), dialog);
 	g_signal_connect (dialog->addto_dialog, "destroy",
 			  G_CALLBACK (panel_addto_dialog_destroy), dialog);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	vbox = gtk_vbox_new (FALSE, 12);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 	gtk_container_add (GTK_CONTAINER (dialog_vbox), vbox);
+#endif
 
 	inner_vbox = gtk_vbox_new (FALSE, 6);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_box_pack_start (GTK_BOX (dialog_vbox), inner_vbox, TRUE, TRUE, 0);
+#else
 	gtk_box_pack_start (GTK_BOX (vbox), inner_vbox, TRUE, TRUE, 0);
+#endif
 
 	find_hbox = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (inner_vbox), find_hbox, FALSE, FALSE, 0);
@@ -1360,7 +1377,11 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 
 	gtk_container_add (GTK_CONTAINER (sw), dialog->tree_view);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_show_all (dialog_vbox);
+#else
 	gtk_widget_show_all (vbox);
+#endif
 
 	panel_toplevel_push_autohide_disabler (dialog->panel_widget->toplevel);
 	panel_widget_register_open_dialog (panel_widget,
