@@ -194,7 +194,11 @@ kill_window_response (GtkDialog *dialog,
 		gdk_error_trap_push ();
 		XKillClient (display, window);
 		gdk_flush ();
+#if GTK_CHECK_VERSION (3, 0, 0)
+		gdk_error_trap_pop_ignored ();
+#else
 		gdk_error_trap_pop ();
+#endif
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -251,7 +255,11 @@ handle_button_press_event (GtkWidget *popup,
 	window = find_managed_window (event->display, event->subwindow);
 
 	if (window != None) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+		if (!gdk_x11_window_lookup_for_display (gdk_x11_lookup_xdisplay (event->display), window))
+#else
 		if (!gdk_xid_table_lookup_for_display (gdk_x11_lookup_xdisplay (event->display), window))
+#endif
 			kill_window_question ((gpointer) window);
 	}
 }
