@@ -359,7 +359,12 @@ panel_menu_button_menu_deactivated (PanelMenuButton *button)
 {
 	panel_toplevel_pop_autohide_disabler (button->priv->toplevel);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_unset_state_flags (GTK_WIDGET (button),
+								  GTK_STATE_FLAG_PRELIGHT);
+#else
 	GTK_BUTTON (button)->in_button = FALSE;
+#endif
 	button_widget_set_ignore_leave (BUTTON_WIDGET (button), FALSE);
 }
 
@@ -579,14 +584,17 @@ panel_menu_button_gsettings_notify (GSettings     *settings,
 									PanelMenuButton *button)
 {
 	if (!strcmp (key, PANEL_OBJECT_MENU_PATH_KEY)) {
-		panel_menu_button_set_menu_path (button,
-							 g_settings_get_string (settings, key));
+		gchar *value = g_settings_get_string (settings, key);
+		panel_menu_button_set_menu_path (button, value);
+		g_free (value);
 	} else if (!strcmp (key, PANEL_OBJECT_CUSTOM_ICON_KEY)) {
-		panel_menu_button_set_custom_icon (button,
-							 g_settings_get_string (settings, key));
+		gchar *value = g_settings_get_string (settings, key);
+		panel_menu_button_set_custom_icon (button, value);
+		g_free (value);
 	} else if (!strcmp (key, PANEL_OBJECT_TOOLTIP_KEY)) {
-		panel_menu_button_set_tooltip (button,
-							 g_settings_get_string (settings, key));
+		gchar *value = g_settings_get_string (settings, key);
+		panel_menu_button_set_tooltip (button, value);
+		g_free (value);
 	} else if (!strcmp (key, PANEL_OBJECT_USE_MENU_PATH_KEY)) {
 		panel_menu_button_set_use_menu_path (button,
 							 g_settings_get_boolean (settings, key));
@@ -979,7 +987,7 @@ panel_menu_button_invoke_menu (PanelMenuButton *button,
 	screen = gtk_widget_get_screen (GTK_WIDGET (button));
 
 	if (!strcmp (callback_name, "help")) {
-		panel_show_help (screen, "user-guide", "gospanel-37", NULL);
+		panel_show_help (screen, "mate-user-guide", "gospanel-37", NULL);
 
 	} else if (!strcmp (callback_name, "edit")) {
                 GError *error = NULL;
