@@ -82,6 +82,7 @@ static GQuark queued_changes_quark = 0;
 static GQuark commit_timeout_quark = 0;
 
 static void panel_profile_object_id_list_update (gchar **objects);
+static void panel_profile_ensure_toplevel_per_screen (void);
 
 static void
 panel_profile_set_toplevel_id (PanelToplevel *toplevel,
@@ -232,20 +233,67 @@ panel_profile_get_background_type (PanelToplevel *toplevel)
 
 void
 panel_profile_set_background_color (PanelToplevel *toplevel,
+#if GTK_CHECK_VERSION (3, 0, 0)
+				    GdkRGBA       *color)
+#else
 				    PanelColor    *color)
+#endif
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	panel_profile_set_background_gdk_rgba_color (toplevel, color);
+#else
 	panel_profile_set_background_gdk_color (toplevel, &color->gdk);
+#endif
 	panel_profile_set_background_opacity (toplevel, color->alpha);
 }
 
 void
 panel_profile_get_background_color (PanelToplevel *toplevel,
+#if GTK_CHECK_VERSION (3, 0, 0)
+				    GdkRGBA       *color)
+#else
 				    PanelColor    *color)
+#endif
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	panel_profile_get_background_gdk_rgba_color (toplevel, color);
+	color->alpha = panel_profile_get_background_opacity (toplevel) / 65535.0;
+#else
 	panel_profile_get_background_gdk_color (toplevel, &(color->gdk));
 	color->alpha = panel_profile_get_background_opacity (toplevel);
+#endif
 }
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+void
+panel_profile_set_background_gdk_rgba_color (PanelToplevel *toplevel,
+					GdkRGBA      *color)
+{
+	char        *color_str;
+
+	color_str = gdk_rgba_to_string (color);
+
+	g_settings_set_string(toplevel->background_settings, "color", color_str);
+
+	g_free (color_str);
+}
+
+void
+panel_profile_get_background_gdk_rgba_color (PanelToplevel *toplevel,
+					GdkRGBA      *color)
+{
+	char        *color_str;
+
+	color_str = g_settings_get_string (toplevel->background_settings, "color");
+	if (!color_str || !gdk_rgba_parse (color, color_str)) {
+		color->red   = 0.;
+		color->green = 0.;
+		color->blue  = 0.;
+	}
+
+	g_free (color_str);
+}
+#else
 void
 panel_profile_set_background_gdk_color (PanelToplevel *toplevel,
 					GdkColor      *gdk_color)
@@ -277,6 +325,7 @@ panel_profile_get_background_gdk_color (PanelToplevel *toplevel,
 
 	g_free (color_str);
 }
+#endif
 
 void
 panel_profile_set_background_opacity (PanelToplevel *toplevel,
@@ -495,18 +544,34 @@ panel_profile_is_writable_attached_tooltip (PanelToplevel *toplevel)
 
 static void
 get_background_color (PanelToplevel *toplevel,
-					  PanelColor  *color)
+#if GTK_CHECK_VERSION (3, 0, 0)
+					  GdkRGBA       *color)
+#else
+					  PanelColor    *color)
+#endif
 {
 	char       *color_str;
 	color_str = g_settings_get_string (toplevel->background_settings, "color");
+#if GTK_CHECK_VERSION (3, 0, 0)
+	if (!color_str || !gdk_rgba_parse (color, color_str)) {
+		color->red   = 0;
+		color->green = 0;
+		color->blue  = 0;
+	}
+#else
 	if (!color_str || !gdk_color_parse (color_str, &(color->gdk))) {
 		color->gdk.red   = 0;
 		color->gdk.green = 0;
 		color->gdk.blue  = 0;
 	}
+#endif
 	g_free (color_str);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	color->alpha = g_settings_get_int (toplevel->background_settings, "opacity") / 65535.0;
+#else
 	color->alpha = g_settings_get_int (toplevel->background_settings, "opacity");
+#endif
 }
 
 static char *
@@ -529,7 +594,11 @@ panel_profile_load_background (PanelToplevel *toplevel)
 	PanelWidget         *panel_widget;
 	PanelBackground     *background;
 	PanelBackgroundType  background_type;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkRGBA              color;
+#else
 	PanelColor           color;
+#endif
 	char                *image;
 	gboolean             fit;
 	gboolean             stretch;
@@ -710,11 +779,14 @@ panel_profile_toplevel_change_notify (GSettings *settings,
 									  gchar *key,
 									  PanelToplevel *toplevel)
 {
+	if (toplevel == NULL || !PANEL_IS_TOPLEVEL (toplevel))
+		return;
 
 #define UPDATE_STRING(k, n)                                                     \
 		if (!strcmp (key, k)) {                                                 \
-			panel_toplevel_set_##n (toplevel,                                   \
-									g_settings_get_string (settings, key));     \
+			gchar *value = g_settings_get_string (settings, key);               \
+			panel_toplevel_set_##n (toplevel, value);                           \
+			g_free (value);                                                     \
 		}
 
 #define UPDATE_ENUM(k, n)                                                       \
@@ -812,6 +884,8 @@ panel_profile_background_change_notify (GSettings *settings,
 	PanelWidget     *panel_widget;
 	PanelBackground *background;
 
+	if (toplevel == NULL || !PANEL_IS_TOPLEVEL (toplevel))
+		return;
 	panel_widget = panel_toplevel_get_panel_widget (toplevel);
 	if (panel_widget == NULL)
 		return;
@@ -823,17 +897,28 @@ panel_profile_background_change_notify (GSettings *settings,
 		panel_background_set_type (background, background_type);
 		panel_toplevel_update_edges (toplevel);
 	} else if (!strcmp (key, "color")) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+		GdkRGBA color;
+		gchar *str;
+		str = g_settings_get_string (settings, key);
+		if (gdk_rgba_parse (&color, str))
+			panel_background_set_color (background, &color);
+		g_free (str);
+#else
 		GdkColor    gdk_color;
-		const char *str;
+		gchar *str;
 		str = g_settings_get_string (settings, key);
 		if (gdk_color_parse (str, &gdk_color))
 			panel_background_set_gdk_color (background, &gdk_color);
+		g_free (str);
+#endif
 	} else if (!strcmp (key, "opacity")) {
 		panel_background_set_opacity (background,
 					      g_settings_get_int (settings, key));
 	} else if (!strcmp (key, "image")) {
-		panel_background_set_image (background,
-					    g_settings_get_string (settings, key));
+		gchar *value = g_settings_get_string (settings, key);
+		panel_background_set_image (background, value);
+		g_free (value);
 	} else if (!strcmp (key, "fit")) {
 		panel_background_set_fit (background,
 					  g_settings_get_boolean (settings, key));
@@ -1516,6 +1601,10 @@ panel_profile_toplevel_id_list_notify (GSettings *settings,
 									  (PanelProfileGetIdFunc) panel_profile_get_toplevel_id,
 									  (PanelProfileDestroyFunc) panel_profile_destroy_toplevel);
 
+	/* if there are no panels, reset layout to default */
+	if (g_slist_length (toplevel_ids) == 0)
+		panel_profile_ensure_toplevel_per_screen ();
+
 	g_slist_free (existing_toplevels);
 	g_slist_free (toplevel_ids);
 }
